"""
    This module provide an easy way to build menu prompts for CLI
    this module contain the next Class:
    Checkboxlist() for create simple and easy useful Menu 

    This module was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
        
    PyeMenu version 1.0.1
"""
# Locals
from ..components import Text, Title, Checkbox, Entry, Button
from ..colors import Colors, setColor
from ..tools import getKeyboard, clear_screen, setCursor, print_title
from ..tools import resize_screen, print_logo, fill_empty_blocks
# Externals
import math
from readchar import key
from copy import deepcopy

nf = '\x1b[0m'
not_fg = '\x1b[39m'
not_bg = '\x1b[49m'
input_info = """
    Press Space to input
    Press Delete to clear the input value 
    Press Enter to exit and return the answers"""

def clear_all(form):
    for entry in form:
        entry.clear()

def exit_form(self, form):
    self.exit = True
    self.answer = {entry.text:' ' for entry in self.entries}
    return self.exit, self.answer

def submit_answer(self):
    self.exit = True
    return self.exit

class Form():
    '''
    This class allow to create an simple option list for selecting
    one option from a list of options.
    The parameters to initialize the class are:
        entries: List -> List of entries for gather answers or info
            could be a list with strins or Entry or Text Type
        buttons: List -> Must be strings or Button object
        title: str or Text object -> The text title to be printed up the menu
        cursor: str or Text object -> The char/s that represent the cursor
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        placeholder_fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        placeholder_bg: str -> A color in hexadecimal, much of colors could be found in Colors class
    Some of the properties of this class are:
        answer -> return the selected answer from the entries as dict
    For Show this Form it should be use the print() method.
    '''
    def __init__(self, entries: list, buttons = ['clear', 'submit', 'exit'],
                title: str = '', cursor: str = '-->',
                fg: str = not_fg, bg: str = not_bg,
                placeholder_fg: str = not_fg, placeholder_bg: str = not_bg):
        self.title = title
        self.cursor = cursor
        self.fg = fg
        self.fg_rgb = '\x1b[38;'+setColor(fg)
        self.bg = bg
        self.bg_rgb = '\x1b[48;'+setColor(bg)
        self.placeholder_fg = placeholder_fg
        self.placeholder_bg = placeholder_bg
        self.placeholder_fg, self.placeholder_bg = Form.setPlaceholder(self)
        self.entries = Form.setEntries(self, entries)
        self.buttons = Form.setButtons(self, buttons)
        self.elements = deepcopy(self.entries)
        self.elements.extend(self.buttons)
        self.max_len_entry = max(self.elements, key = lambda x: x.lenght).lenght
        self.max_len_button = max(self.buttons, key = lambda x: x.lenght).lenght
        self.max_len_values = max(self.entries, key = lambda x: x._len_value)._len_value
        self.answer = {entry.text:entry.value for entry in self.entries}
        self.cursor = Text.setText(self, cursor)
        self.title = Title.setTitle(self, title)
        self.title.width = self.max_len_entry
        self.exit = False
        if buttons == ['clear', 'submit', 'exit']:
            self.buttons[0].onClick = lambda: clear_all(self.entries)
            self.buttons[1].onClick = lambda: submit_answer(self)
            self.buttons[2].onClick = lambda: exit_form(self, self.entries)
        
    def print(self,
            wrap: int=1,
            highlight: bool = True,
            fg_hl = Colors.Azure,
            bg_hl = Colors.Navy, 
            title_decorator: str= ' ',
            title_align: str='center',
            padding_up: bool = False,
            padding_bottom: bool = False, 
            title_padding_up: bool = False,
            title_padding_bottom: bool = False, 
            button_focus_blink: bool = False,
            logo: str = ''
            ):
        """
        This Method allow a Form to be show on screen, it is possible to 
        specifies:
        parameters:
            wrap: int -> how elements into options are wrapped
            highlight: bool -> True by default
            fg_hl -> foreground color for highlight current option
            bg_hl -> background color for highlight current option
            title_align: str -> 'center', 'right' or 'left'
            title_decorator: str -> just one char to print around Title text
            padding_up: bool = False -> add a new line above title
            padding_bottom: bool = False -> add a new line behind title
            title_padding_up: bool = False -> add a new line above title
            title_padding_bottom: bool = False -> add a new line behind title
            logo: str -> This parameter allow to print a logo or an extra title 
                above the Form print
        """
        # Initialazing Variables
        pointer = 0
        keyboard = None
        while True:
            clear_screen()
            self.answer = {entry.text:entry.value for entry in self.entries}
            if keyboard == key.DELETE:
                if pointer in range(len(self.entries)):
                    clear_screen()
                    self.entries[pointer].clear()
                    clear_screen()
            if keyboard == key.SPACE:
                if pointer in range(len(self.entries)):
                    clear_screen()
                    self.entries[pointer].onSelect()
                    clear_screen()
            if keyboard == key.ENTER:
                if pointer in range(len(self.entries)):
                    clear_screen()
                    self.entries[pointer].onSelect()
                    clear_screen()
                if pointer in range(len(self.entries), len(self.elements)):
                    clear_screen()
                    self.elements[pointer].click()
                    clear_screen()
            if keyboard in ["q", "Q"]:
                self.answer = {entry.text:'' for entry in self.entries}
                break
            if self.exit == True:
                break
            
            
            self.max_len_values = max(self.entries, key = lambda x: x._len_value)._len_value
            block_width = 7+self.cursor.lenght+self.max_len_entry+self.max_len_values
            wrap = resize_screen(wrap, block_width)
            print_logo(logo)

            if padding_up:
                print(f"{self.bg_rgb}{((block_width)*wrap)*' '}")
            print_title(self, title_align, title_decorator, 
                        block_width, wrap, title_padding_up, 
                        title_padding_bottom)
            for entry in self.entries:
                if self.entries.index(entry) % wrap == 0:
                    print("")
                if pointer == self.entries.index(entry):
                    if highlight:
                        it_hl = Entry(entry.text, entry.value, entry.validation,
                                    fg_hl, bg_hl, fg_hl, bg_hl, entry.bold, 
                                    entry.italic, entry.underline, entry.blink, entry.reverse, entry.crossed)
                        if it_hl.validation=='password':
                            print(
                                f"{self.cursor.bg_rgb} {self.cursor.styled}{self.cursor.bg_rgb} "\
                                +f"{it_hl.styled_text}"\
                                +f"{it_hl.bg_rgb}{(self.max_len_entry-entry.lenght)*' '}"\
                                +f"{it_hl.fg_rgb}:"\
                                +f" {it_hl._len_value*'*'} "\
                                +f"{it_hl.placeholder_bg_rgb}{(self.max_len_values-entry._len_value)*' '}",
                                end="")
                        else:
                            print(
                                f"{self.cursor.bg_rgb} {self.cursor.styled}{self.cursor.bg_rgb} "\
                                +f"{it_hl.styled_text}"\
                                +f"{it_hl.bg_rgb}{(self.max_len_entry-entry.lenght)*' '}"\
                                +f"{it_hl.fg_rgb}:"\
                                +f"{it_hl.styled_value}"\
                                +f"{it_hl.placeholder_bg_rgb}{(self.max_len_values-entry._len_value)*' '}",
                                end="")
                    else:
                        print(
                            f"{self.cursor.bg_rgb} {self.cursor.styled}{self.cursor.bg_rgb} "\
                            +f"{entry.styled_text}"\
                            +f"{entry.bg_rgb}{(self.max_len_entry-entry.lenght)*' '}"\
                            +f"{entry.fg_rgb}{entry.bg_rgb}:"\
                            +f"{entry.styled_value}{entry.placeholder_fg_rgb}"\
                            +f"{entry.placeholder_bg_rgb}{(self.max_len_values-entry._len_value)*' '}", 
                            end="")
                else:
                    print(
                        f"{self.cursor.bg_rgb} {' '*(len(self.cursor.text))}{self.cursor.bg_rgb} "\
                        +f"{entry.styled_text}"\
                        +f"{entry.bg_rgb}{(self.max_len_entry-entry.lenght)*' '}"\
                        +f"{entry.fg_rgb}{entry.bg_rgb}:"\
                        +f"{entry.styled_value}{entry.placeholder_fg_rgb}"\
                        +f"{entry.placeholder_bg_rgb}{(self.max_len_values-entry._len_value)*' '}", 
                        end="")

            empty_blocks = int(math.ceil(len(self.entries)/wrap)*wrap)-len(self.entries)
            fill_empty_blocks(self, empty_blocks, block_width)
            if padding_bottom:
                print(f"\n{self.bg_rgb}{((block_width)*wrap)*' '}")
            
            print(f"{nf}")
            print(f"{block_width*wrap*'—'}", end='')
            for button in self.buttons:
                button_space = 3+self.max_len_values+self.max_len_entry-button.lenght
                if button_space%2==0:
                    left_spaces = button_space//2
                    right_spaces = left_spaces
                else:
                    left_spaces = button_space//2
                    right_spaces = button_space//2 + 1

                if self.buttons.index(button) % wrap == 0:
                    print("\n")
                
                if pointer == self.elements.index(button):
                    button_selected = Button(label=button.text, bg=bg_hl, fg=fg_hl, blink=button_focus_blink)
                    print(
                        f" {' '*(len(self.cursor.text))} "\
                        +f"{button_selected.bg_rgb}{(left_spaces)*' '}"\
                        +f"{button_selected.print}"\
                        +f"{button_selected.bg_rgb}{(right_spaces)*' '}\x1b[0m"\
                        ,end='')
                else:
                    print(
                        f" {' '*(len(self.cursor.text))} "\
                        +f"{button.bg_rgb}{(left_spaces)*' '}"\
                        +f"{button.print}"\
                        +f"{button.bg_rgb}{(right_spaces)*' '}\x1b[0m"\
                        ,end='')
            
            print(f"{nf}")
            keyboard = getKeyboard(input_info)
            pointer = setCursor(keyboard, pointer, self.elements, wrap)

    def setEntries(self, entries):
        __entries = []
        for entry in entries:   
            if type(entry) in [type(Text('')), type(Entry('')), type(Checkbox(''))]:
                if entry.bg == not_bg and entry.fg == not_fg:
                    entry.bg = self.bg
                    entry.fg = self.fg
                elif entry.bg != not_bg and entry.fg == not_fg:
                    entry.fg = self.fg
                elif entry.bg == not_bg and entry.fg != not_fg:
                    entry.bg = self.bg

                if type(entry) == type(Entry('')):
                    if entry.placeholder_bg == not_bg and entry.placeholder_fg == not_fg:
                        entry.placeholder_bg = self.placeholder_bg
                        entry.placeholder_fg = self.placeholder_fg
                    elif entry.placeholder_bg != not_bg and entry.placeholder_fg == not_fg:
                        entry.placeholder_fg = self.placeholder_fg
                    elif entry.placeholder_bg == not_bg and entry.placeholder_fg != not_fg:
                        entry.placeholder_bg = self.placeholder_bg
                        
                    entry = Entry(entry.text, entry.value, entry.validation,
                                entry.fg, entry.bg, entry.placeholder_fg, entry.placeholder_bg, 
                                entry.bold, entry.italic, 
                                entry.underline, entry.blink, entry.reverse, entry.crossed)
                    
                if type(entry) == type(Text('')):
                    entry = Entry(entry.text, '', 'all', 
                                        self.fg, self.bg, 
                                        self.placeholder_fg, self.placeholder_bg, 
                                        entry.bold, entry.italic, 
                                        entry.underline, entry.blink, 
                                        entry.reverse, entry.crossed)
                    
                if type(entry) == type(Checkbox('')):
                    entry = Entry(entry.text, value='', validation='checkbox', 
                                    fg=entry.fg, bg=entry.bg, 
                                    placeholder_fg=self.placeholder_fg, 
                                    placeholder_bg=self.placeholder_bg, 
                                    bold=entry.bold, italic=entry.italic, 
                                    underline=entry.underline, blink=entry.blink, 
                                    reverse=entry.reverse, crossed=entry.crossed)
            else:
                entry = Entry(str(entry), value='', validation='all',fg=self.fg, bg=self.bg, 
                                placeholder_fg=self.placeholder_fg, placeholder_bg=self.placeholder_bg)        
                
            __entries.append(entry)

        return __entries

    def setButtons(self, buttons):
        __buttons = []
        for button in buttons:   
            if type(button) != type(Button(' ')):
                button = Button(str(button))        
                
            __buttons.append(button)

        return __buttons
    
    def setPlaceholder(self):
        if self.placeholder_bg == not_bg and self.placeholder_fg == not_fg:
            placeholder_fg_rgb = self.fg
            placeholder_bg_rgb = self.bg
        if self.placeholder_bg != not_bg and self.placeholder_fg == not_fg:
            placeholder_fg_rgb = self.fg
            placeholder_bg_rgb = self.placeholder_bg
        if self.placeholder_bg == not_bg and self.placeholder_fg != not_fg:
            placeholder_fg_rgb = self.placeholder_fg
            placeholder_bg_rgb = self.bg
        if self.placeholder_bg != not_bg and self.placeholder_fg != not_fg:
            placeholder_fg_rgb = self.placeholder_fg
            placeholder_bg_rgb = self.placeholder_bg
        
        return placeholder_fg_rgb, placeholder_bg_rgb


import re
from .texts import Text
from ..colors import setColor

import getpass

not_fg = '\x1b[39m'
not_bg = '\x1b[49m'
email_regex = r'([A-Za-z0-9]+[.-_])*[A-Za-z0-9]+@[A-Za-z0-9-]+(\.[A-Z|a-z]{2,})+' 

class Entry(Text):
    """
    Generate a Entry with its label and its value, this class extend from Text()
    this class allow to perform a validition from user's input, there are a predefined
    validitaion types, this must be pass as string, for example validation='all'
    these are:
        1. password -> allow to get an password value and show hidden in the form
        2. all      -> allow anything from user, this is the value for validation by default
        3. int      -> allow just integer
        4. alnum    -> allow just alphanumeric chars
        5. email    -> allow just a valid email format
        6. checkbox -> allow the entry to perform as a Checkbox
        
        Also it is possible to pass a validation function for validate the desire input from user
        these must return just False or True bool type
        
    some formats and effects are not available
    in some terminal emulators as Kitty, Alacritty, etc...
    A full list of terminals will be show on https://github.com/FreireAlexander/PyeMenu
    properties:
        text: str
        value: str=''
        validation: str='all'         
    ***** Text Style *****
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        placeholder_fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        placeholder_bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        bold: bool 
        italic: bool 
        underline: bool 
        blink: bool 
        reverse: bool 
        crossed: bool
        
    PyeMenu version 1.0.1
    """
    def __init__(self, label: str, value: str='', validation='all',
                fg: str = not_fg, bg: str = not_bg,
                placeholder_fg: str = not_fg, placeholder_bg: str = not_bg,
                bold: bool = False, italic: bool = False, underline: bool = False,
                blink: bool = False, reverse: bool = False, crossed: bool = False):
        super().__init__(label, fg, bg,
                            bold, italic, underline, 
                            blink, reverse, crossed)
        self.validation = validation
        self._text = ' '+label+' '
        self._lenght = len(self._text)
        self.placeholder_fg = placeholder_fg
        self.placeholder_bg = placeholder_bg
        Entry.setPlaceholder(self)     
        self.styled_text = Text.setStyle(self, self._text, self.fg, self.bg, bold, italic, underline, blink, reverse, crossed)
        self.value = value
        self._value = ' '+value+' '
        self.styled_value = Text.setStyle(self, self._value, self.placeholder_fg, self.placeholder_bg, bold, italic, underline, blink, reverse, crossed)
        self._len_value = len(self.value)
        self.print_label = f"{self.styled_text}{self.bg_rgb} "+"\x1b[0m"
        self.print_value = f"{self.styled_value}{self.placeholder_bg_rgb}"+"\x1b[0m"
        self.print = self.print_label + f"{self.bg_rgb}{self.fg_rgb}: " + self.print_value
        
    def setPlaceholder(self):
        """
        This method set the placeholder style
        """
        if self.placeholder_bg == not_bg and self.placeholder_fg == not_fg:
            self.placeholder_fg_rgb = '\x1b[38;'+setColor(self.fg)
            self.placeholder_bg_rgb = '\x1b[48;'+setColor(self.bg)
        elif self.placeholder_bg != not_bg and self.placeholder_fg == not_fg:
            self.placeholder_fg_rgb = '\x1b[38;'+setColor(self.fg)
            self.placeholder_bg_rgb = '\x1b[48;'+setColor(self.placeholder_bg)
        elif self.placeholder_bg == not_bg and self.placeholder_fg != not_fg:
            self.placeholder_fg_rgb = '\x1b[38;'+setColor(self.placeholder_fg)
            self.placeholder_bg_rgb = '\x1b[48;'+setColor(self.bg)
        elif self.placeholder_bg != not_bg and self.placeholder_fg != not_fg:
            self.placeholder_fg_rgb = '\x1b[38;'+setColor(self.placeholder_fg)
            self.placeholder_bg_rgb = '\x1b[48;'+setColor(self.placeholder_bg)

        return self.placeholder_fg_rgb, self.placeholder_bg_rgb
    
    
    def onSelect(self):
        """
        This method perform the validation type for the Entry
        """
        if self.validation == 'password':
            try:
                value = getpass.getpass(prompt='Type Password: ')
            except Exception as error:
                print('ERROR', error)
            self.value = value
            self._value = " "+f"{len(value)*'*'}"+" "
            self._len_value = len(self.value)
            self.styled_value = Text.setStyle(self, self._value, self.placeholder_fg, self.placeholder_bg, self.bold, 
                                        self.italic, self.underline, self.blink, self.reverse, self.crossed)
            self.print_value = f"{self.styled_value}{self.placeholder_bg_rgb}"+"\x1b[0m"
            self.print = self.print_label + f"{self.bg_rgb}{self.fg_rgb}: " + self.print_value
        else:
            if self.validation == 'all':
                value = input("Type new value: ")
            elif self.validation == 'float':
                while True:
                    try:
                        value = str(float(input("Please enter a number: ")))
                    except ValueError:
                        print("Oops!  That was not a valid number.  Try again...")
                    else:
                        break
            elif self.validation == 'int':
                while True:
                    try:
                        value = str(int(input("Please enter a number: ")))
                    except ValueError:
                        print("Oops!  That was not a valid integer number.  Try again...")
                    else:
                        break
            elif self.validation == 'alnum':
                value = input("Type an alphanumeric value: ")
                while value.isalnum() == False:
                    print("Oop's somethings is wrong... try again")
                    value = input("Type an alphanumeric value: ")
            elif self.validation == 'email':
                value = input("Type email: ")
                while re.match(email_regex, value) == None:
                    print("OOps it's not email")
                    value = input("Type email again: ")
            elif self.validation == 'checkbox':
                if self.value == '':
                    value = '*'
                else:
                    value = ''
            else:
                value = input("Type new value: ")
                while self.validation(value) == False:
                    print("Oop's somethings is wrong... try again")
                    value = input("Type new value: ")

            self.value = value
            self._len_value = len(self.value)
            self._value = ' '+str(value)+' '
            self.styled_value = Text.setStyle(self, self._value, self.placeholder_fg, self.placeholder_bg, self.bold, 
                                        self.italic, self.underline, self.blink, self.reverse, self.crossed)
            self.print_value = f"{self.styled_value}{self.placeholder_bg_rgb}"+"\x1b[0m"
            self.print = self.print_label + f"{self.bg_rgb}{self.fg_rgb}: " + self.print_value
    
    def clear(self):
        """
        This method clear the value of the Entry
        """
        self.value = ''
        self._len_value = len(self.value)
        self._value = ' '+str(self.value)+' '
        self.styled_value = Text.setStyle(self, self._value, self.placeholder_fg, 
                                            self.placeholder_bg, self.bold, 
                                            self.italic, self.underline, self.blink, 
                                            self.reverse, self.crossed)
        self.print_value = f"{self.styled_value}{self.placeholder_bg_rgb}"+"\x1b[0m"
        self.print = self.print_label + f"{self.bg_rgb}{self.fg_rgb}: " + self.print_value


"""
    This module provide an easy way to build menu prompts for CLI
    this module contain the next Class:
    Checkboxlist() for create simple and easy useful Menu 

    This module was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
        
    PyeMenu version 1.0.1
"""
# Locals
from ..components import Text, Title, Checkbox
from ..colors import Colors, setColor
from ..tools import setCursor, getKeyboard, clear_screen, resize_screen
from ..tools import print_title, fill_empty_blocks, print_logo
# External
import math
from readchar import key


nf = '\x1b[0m'
not_fg = '\x1b[39m'
not_bg = '\x1b[49m'
input_info = """
    Press Space to select or unselect an option
    Press Enter to exit and return the choice/s"""

class Checkboxlist():
    '''
    This class allow to create an simple option list for selecting
    one option from a list of options.
    The parameters to initialize the class are:
        options: List -> List of labels for been selected
        multiselect: bool -> Allow multiple choices by default it is True
        title: str or Text object -> The text title to be printed up the menu
        cursor: str or Text object -> The char/s that represent the cursor
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
    Some of the properties of this class are:
        answer -> return the selected value from the list
    For Show this Checkboxlist it should be use the print() method. 
    '''
    def __init__(self, options: list,
                multiselect: bool = True, 
                title: str = '', cursor: str = '-->',
                fg: str = not_fg, bg: str = not_bg):
        self.multiselect = multiselect
        __options = [option if type(option)==type(Text('')) else Checkbox(str(option)) for option in options]
        self.max_len_option = len(max(__options, key = lambda x: x.lenght).text)
        self.title = title
        self.cursor = cursor
        self.fg = fg
        self.fg_rgb = '\x1b[38;'+setColor(fg)
        self.bg = bg
        self.bg_rgb = '\x1b[48;'+setColor(bg)
        self.answer = []
        self.cursor = Text.setText(self, cursor)
        self.title = Title.setTitle(self, title)
        self.options = self.setOptions(options)
        self.title.width = self.max_len_option
        
    def print(self,
            wrap: int=1,
            highlight: bool = True,
            fg_hl = Colors.white,
            bg_hl = Colors.Lime, 
            title_decorator: str= ' ',
            title_align: str='center',
            padding_up: bool = False,
            padding_bottom: bool = False, 
            title_padding_up: bool = False,
            title_padding_bottom: bool = False,
            logo: str = ''
            ):
        """
        This Method allow a Checkboxlist to be show on screen
        Return a string with the selected values or None
        parameters:
            wrap: int -> how elements into options are wrapped
            highlight: bool -> True by default
            fg_hl -> foreground color for highlight current option
            bg_hl -> background color for highlight current option
            title_align: str -> 'center', 'right' or 'left'
            title_decorator: str -> just one char to print around Title text
            padding_up: bool = False -> add a new line above title
            padding_bottom: bool = False -> add a new line behind title
            title_padding_up: bool = False -> add a new line above title
            title_padding_bottom: bool = False -> add a new line behind title
            logo: str -> This parameter allow to print a logo or an extra title 
                above the Form print
        """
        pointer = 0
        self.answer = []
        keyboard = None
        while True:
            clear_screen()
            block_width = 9+self.cursor.lenght+self.max_len_option
            wrap = resize_screen(wrap, block_width)
            print_logo(logo)
            if padding_up:
                print(f"{self.bg_rgb}{((block_width)*wrap)*' '}")
            print_title(self, title_align, title_decorator, 
                        block_width, wrap, title_padding_up, 
                        title_padding_bottom)
            if keyboard == key.SPACE:
                if self.multiselect:
                    if self.options[pointer].text not in self.answer:
                        self.options[pointer].onSelect() 
                        self.answer.append(self.options[self.options.index(self.options[pointer])].text)
                    elif self.options[pointer].text in self.answer:
                        self.options[pointer].onSelect()
                        self.answer.remove(self.options[self.options.index(self.options[pointer])].text)
                else:
                    self.options[pointer].onSelect()
                    for other in self.options:
                        if other != self.options[pointer]:
                            other.clear()
                            self.answer = self.options[self.options.index(self.options[pointer])].text
            for option in self.options:
                if self.options.index(option) % wrap == 0:
                    print("")
                if pointer == self.options.index(option):
                    if highlight:
                        it_hl = Checkbox(option.text,  box=option.box,
                                        fg=fg_hl, bg=bg_hl,
                                        bold=option.bold, italic=option.italic, 
                                        underline=option.underline, 
                                        blink=option.blink, reverse=option.reverse, 
                                        crossed=option.crossed)
                        print(
                            f"{self.cursor.bg_rgb} {self.cursor.styled}{self.cursor.bg_rgb} "\
                            +f"{it_hl.print}{it_hl.bg_rgb}"\
                            +f"{(self.max_len_option-len(option.text))*' '}", 
                            end="")
                    else:
                        print(
                            f"{self.cursor.bg_rgb} {self.cursor.styled}{self.cursor.bg_rgb} "\
                            +f"{option.print}{option.bg_rgb}"\
                            +f"{(self.max_len_option-len(option.text))*' '}", 
                            end="")    
                else:
                    print(
                        f"{self.cursor.bg_rgb} {' '*(len(self.cursor.text))}{self.cursor.bg_rgb} "\
                        +f"{option.print}{option.bg_rgb}"\
                        +f"{(self.max_len_option-len(option.text))*' '}", 
                        end="")
            empty_blocks = int(math.ceil(len(self.options)/wrap)*wrap)-len(self.options)
            fill_empty_blocks(self, empty_blocks, block_width)
            if padding_bottom:
                print(f"\n{self.bg_rgb}{(block_width*wrap)*' '}")
            print(f"{nf}")
            keyboard = getKeyboard(info=input_info)
            pointer = setCursor(keyboard, pointer, self.options, wrap)
            if keyboard in ["q", "Q"]:
                self.answer = None
                return self.answer
            if keyboard == key.ENTER:
                return self.answer

    def setOptions(self, options):
        list_options = []
        for option in options:
            if type(option) != type(Text('')):
                option = Checkbox(str(option), fg=self.fg, bg=self.bg)
            if type(option) == type(Text('')) and option.bg != not_bg and option.fg != not_fg:
                option = Checkbox(option.text, fg=option.fg, bg=option.bg,
                                bold=option.bold, italic=option.italic, underline=option.underline, 
                                blink=option.blink, reverse=option.reverse, crossed=option.crossed)
            if type(option) == type(Text('')) and option.bg == not_bg and option.fg != not_fg:
                option = Checkbox(option.text, bg=self.bg, fg=option.fg,
                                bold=option.bold, italic=option.italic, underline=option.underline, 
                                blink=option.blink, reverse=option.reverse, crossed=option.crossed)
            if type(option) == type(Text('')) and option.fg == not_fg and option.bg != not_bg:
                option = Checkbox(option.text, fg=self.fg, bg=option.bg,
                                bold=option.bold, italic=option.italic, underline=option.underline, 
                                blink=option.blink, reverse=option.reverse, crossed=option.crossed)
            if type(option) == type(Text('')) and option.fg == not_fg and option.bg == not_bg:
                option = Checkbox(option.text, fg=self.fg, bg=self.bg,
                                bold=option.bold, italic=option.italic, underline=option.underline, 
                                blink=option.blink, reverse=option.reverse, crossed=option.crossed)
            
            list_options.append(option)
        return list_options



"""
    This module provide specific tools for generate menus
    this module contain the next functions:
    0. clear_screen  : Clean the screen
    1. getKeyboard : read key pressed

    This module was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*

    PyeMenu version 1.0.1
"""
#Libraries
import os
from os import get_terminal_size
from readchar import key, readkey
from .components import Text, Title

## Global Variables
# Keyboard Values
ENTER   = key.ENTER
UP      = key.UP
DOWN    = key.DOWN
LEFT    = key.LEFT
RIGHT   = key.RIGHT
SPACE   = key.SPACE
input_info = """
    Press ENTER to select an option"""
input_exit = """
    Press (q) or (Q) to exit"""

# Os.clear functions
def clear_screen():
    """
    Clean windows screen
    """
    if os.name == "nt":
        os.system("cls")
    else:
        os.system("clear")

# input key
def getKeyboard(info:str=input_info, exit_message:str=input_exit):
    """
    This Function read the key press from user
    Also Allow to introduce an info messages and
    an exit message, by default:
    info = Press ENTER to select an option}
    exit_message = Press (q) to exit
    params
    info: str -> information of keyboard functions for the user 
    exit_message: str -> information of keyboard input to exit for the user
    """
    print(f"{info}", end="")
    print(f"{exit_message}")
    return readkey()

def setCursor(keyboard, pointer: int, options: list, wrap: int):
    """
    This Function allow to place the cursor into a block,
    but it requires
    params
    keyboard: readkey() -> the input from the keyboard
    pointer:    int -> Pointer index where the cursor is placed
    options:  list -> the list of elements in the block
    wrap:     int -> the number of columns of how elements are wrapped
    """
    previous_pointer = pointer
    if keyboard == UP:    pointer -=wrap
    if keyboard == DOWN:  pointer +=wrap
    if keyboard == LEFT:  pointer -=1
    if keyboard == RIGHT: pointer +=1
    if pointer < 0: pointer = previous_pointer
    if previous_pointer > len(options) - 1: pointer = 0
    if pointer > len(options) - 1: pointer = previous_pointer
    
    return pointer

def resize_screen(wrap, block_width):
    """
    This method is for resizing the screen when widgets are print or show
    """
    cols, rows = get_terminal_size()
    while block_width*wrap>cols:
        wrap -=1       
    if wrap <0:
        wrap = 1
    return wrap

def print_title(widget, title_align, title_decorator, 
                block_width, wrap, title_padding_up, title_padding_bottom):
    """
    This method is for print the title in widgets
    """
    widget.title.print_title(title_align, title_decorator, 
                    (block_width)*wrap, 
                    title_padding_up, 
                    title_padding_bottom)

def print_logo(logo):
    """
    This method is for print the logo above widgets
    """
    if type(logo) in [type(Text("")), type(Title(""))]:
        print(f"{logo.styled}")
    elif str(logo):
        logo = Text(str(logo))
        print(f"{logo.styled}")
    else:
        pass

def fill_empty_blocks(self, empty_blocks, block_width):
    """
    This method is for fill empty spaces in widgets when it prints
    """
    if empty_blocks != 0:
        for i in range(empty_blocks):
            print(f"{self.bg_rgb}{((block_width))*' '}", end='')




"""
    This module contain the color value.

    This module was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*

    PyeMenu version 1.0.1
"""
not_fg = '\x1b[39m'
not_bg = '\x1b[49m'

def setColor(color: str):
    """
    This function allow to transform html color to the format needed
    """
    if color == not_fg:
        return not_fg
    elif color == not_bg:
        return not_bg
    else:
        hex = color.lstrip('#')
        color = ";".join([str(int(hex[i:i+2], 16)) for i in (0, 2, 4)])
        color = "2;"+color+"m"
        return color

def html_rgb_fg(color: str):
    """
    This function allow to transform html color to the format needed
    """
    if color == not_fg:
        return not_fg
    else:
        hex = color.lstrip('#')
        color = ";".join([str(int(hex[i:i+2], 16)) for i in (0, 2, 4)])
        color = "\x1b[38;2;"+color+"m"
        return color

def html_rgb_bg(color: str):
    """
    This function allow to transform html color to the format needed
    """
    if color == not_bg:
        return not_bg
    else:
        hex = color.lstrip('#')
        color = ";".join([str(int(hex[i:i+2], 16)) for i in (0, 2, 4)])
        color = "\x1b[48;2;"+color+"m"
        return color


class Colors:
    """
    This Class contains a large list with html colors by its name
    Color By Names from https://htmlcolorcodes.com/color-names/
    """
    white=                       '#ffffff'
    black=                       '#000000'
    gray=                        '#696969'
    yellow=                      '#ffff00'
    red=                         '#ff0000'
    green=                       '#008000'
    blue=                        '#0000ff'
    orange=                      '#ffa500'
    purple=                      '#800080'
    brown=                       '#a52a2a'
    pink=                        '#ffc0cb'
    # Red HTML Color Names 
    IndianRed=                   '#CD5C5C'
    LightCoral=                	 '#F08080'
    Salmon=     	             '#FA8072'
    DarkSalmon=                  '#E9967A'	
    LightSalmon=                 '#FFA07A'	
    Crimson=                     '#DC143C'	 
    FireBrick=                   '#B22222'	
    DarkRed=                     '#8B0000'	
    # Pink HTML Color Names 
    LightPink=	                 '#FFB6C1'
    HotPink=                     '#FF69B4'
    DeepPink=	                 '#FF1493'
    MediumVioletRed=             '#C71585'
    PaleVioletRed=	             '#DB7093'
    # Orange HTML Color Names 
    LightSalmon=                '#FFA07A'	
    Coral=                      '#FF7F50'	
    Tomato=                     '#FF6347'	
    OrangeRed=                  '#FF4500'	
    DarkOrange=                 '#FF8C00'
    # Yellow HTML Color Names 
    Gold=                   	'#FFD700'
    LightYellow=                '#FFFFE0'
    LemonChiffon=               '#FFFACD'
    LightGoldenrodYellow=       '#FAFAD2'
    PapayaWhip=                 '#FFEFD5'
    Moccasin=                   '#FFE4B5'
    PeachPuff=                  '#FFDAB9'
    PaleGoldenrod=              '#EEE8AA'
    Khaki=                  	'#F0E68C'
    DarkKhaki=                  '#BDB76B'
    # Purple HTML Color Names 
    Lavender=	                '#E6E6FA'
    Thistle=	                '#D8BFD8'
    Plum=	                    '#DDA0DD'
    Violet=	                    '#EE82EE'
    Orchid=	                    '#DA70D6'
    Fuchsia=	                '#FF00FF'
    Magenta=	                '#FF00FF'
    MediumOrchid=	            '#BA55D3'
    MediumPurple=	            '#9370DB'
    RebeccaPurple=	            '#663399'
    BlueViolet=	                '#8A2BE2'
    DarkViolet=	                '#9400D3'
    DarkOrchid=	                '#9932CC'
    DarkMagenta=	            '#8B008B'
    Indigo=	                    '#4B0082'
    SlateBlue=	                '#6A5ACD'
    DarkSlateBlue=	            '#483D8B'
    MediumSlateBlue=	        '#7B68EE'
    # Green HTML Color Names 
    GreenYellow=                '#ADFF2F'
    Chartreuse=                 '#7FFF00'
    LawnGree=                   '#7CFC00'
    Lime=                       '#00FF00'
    LimeGreen=                  '#32CD32'
    PaleGreen=                  '#98FB98'
    LightGreen=                 '#90EE90'
    MediumSpringGreen=          '#00FA9A'
    SpringGreen=                '#00FF7F'
    MediumSeaGreen=             '#3CB371'
    SeaGreen=                   '#2E8B57'
    ForestGreen=                '#228B22'
    DarkGreen=                  '#006400'
    YellowGreen=                '#9ACD32'
    OliveDrab=                  '#6B8E23'
    Olive=                      '#808000'
    DarkOliveGreen=             '#556B2F'
    MediumAquamarine=           '#66CDAA'
    DarkSeaGreen=               '#8FBC8B'
    LightSeaGreen=              '#20B2AA'
    DarkCyan=                   '#008B8B'
    Teal=                       '#008080'
    # Blue HTML Color Names 
    Cyan=                   	'#00FFFF'
    LightCyan=                  '#E0FFFF'
    PaleTurquoise=              '#AFEEEE'
    Aquamarine=                 '#7FFFD4'
    Turquoise=                  '#40E0D0'
    MediumTurquoise=            '#48D1CC'
    DarkTurquoise=              '#00CED1'
    CadetBlue=                  '#5F9EA0'
    SteelBlue=                  '#4682B4'
    LightSteelBlue=             '#B0C4DE'
    PowderBlue=                 '#B0E0E6'
    LightBlue=                  '#ADD8E6'
    SkyBlue=                    '#87CEEB'
    LightSkyBlue=               '#87CEFA'
    DeepSkyBlue=                '#00BFFF'
    DodgerBlue=                 '#1E90FF'
    CornflowerBlue=             '#6495ED'
    MediumSlateBlue=            '#7B68EE'
    RoyalBlue=                  '#4169E1'
    MediumBlue=                 '#0000CD'
    DarkBlue=                   '#00008B'
    Navy=                   	'#000080'
    MidnightBlue=               '#191970'
    # Brown HTML Color Names 
    Cornsilk=                   '#FFF8DC'
    BlanchedAlmond=             '#FFEBCD'
    Bisque=                 	'#FFE4C4'
    NavajoWhite=                '#FFDEAD'
    Wheat=                  	'#F5DEB3'
    BurlyWood=                  '#DEB887'
    Tan=                    	'#D2B48C'
    RosyBrown=                  '#BC8F8F'
    SandyBrown=                 '#F4A460'
    Goldenrod=                  '#DAA520'
    DarkGoldenrod=              '#B8860B'
    Peru=                   	'#CD853F'
    Chocolate=                  '#D2691E'
    SaddleBrown=                '#8B4513'
    Sienna=                 	'#A0522D'
    Maroon=                 	'#800000'
    #  White HTML Color Names 
    Snow=                   	'#FFFAFA'
    HoneyDew=                   '#F0FFF0'
    MintCream=                  '#F5FFFA'
    Azure=                  	'#F0FFFF'
    AliceBlue=                  '#F0F8FF'
    GhostWhite=                 '#F8F8FF'
    WhiteSmoke=                 '#F5F5F5'
    SeaShell=                   '#FFF5EE'
    Beige=                  	'#F5F5DC'
    OldLace=                    '#FDF5E6'
    FloralWhite=                '#FFFAF0'
    Ivory=                  	'#FFFFF0'
    AntiqueWhite=               '#FAEBD7'
    Linen=                  	'#FAF0E6'
    LavenderBlush=              '#FFF0F5'
    MistyRose=                  '#FFE4E1'
    # Gray HTML Color Names 
    Gainsboro=                  '#DCDCDC'
    LightGray=                  '#D3D3D3'
    Silver=                 	'#C0C0C0'
    DarkGray=                   '#A9A9A9'
    DimGray=                    '#696969'
    LightSlateGray=             '#778899'
    SlateGray=                  '#708090'
    DarkSlateGray=              '#2F4F4F'
    # Blacks HTML Color Names
    BlackOlive=                 '#3B3C36'
    Onyx=                       '#353839'
    Jet=                        '#343434'
    JetBlack=                   '#0E0E10'


from .texts import Text

not_fg = '\x1b[39m'
not_bg = '\x1b[49m'

class Title(Text):
    """
    This class allow you to format a Title, this class extend from Text()
    some colors styles and effects are not available
    in some terminal emulators as Kitty, Alacritty, etc...
    A full list of terminals will be show on https://github.com/FreireAlexander/PyeMenu
    properties:
        text: str         
    ***** Text Style *****
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        bold: bool 
        italic: bool 
        underline: bool 
        blink: bool 
        reverse: bool 
        crossed: bool 
        
        PyeMenu version 1.0.1
    """
    def __init__(self, text: str,
                fg: str = not_fg, bg: str = not_bg,
                bold: bool = False, italic: bool = False, underline: bool = False,
                blink: bool = False, reverse: bool = False, crossed: bool = False):
        super().__init__(text, fg, bg,
                            bold, italic, underline, 
                            blink, reverse, crossed)
        self.title_text = ' '+text+' '
        self._lenght = len(self.title_text)
        self.styled = Text.setStyle(self, self.title_text, self.fg, self.bg, 
                                    self.bold, self.italic, self.underline, 
                                    self.blink, self.reverse, self.crossed)
        
    
    def print_title(self, align: str = 'center',
                decorator: str = ' ',
                width: int = 0,
                padding_up: bool = False,
                padding_bottom: bool = False):
        """
        This Methods print the Title, it is possible to 
        specifies:
        align: str -> 'center', 'right' or 'left'
        decorator: str -> just one char to print around Title text
        width: int -> width in number of chars
        new_line_up: bool = False -> add a new line above title
        new_line_bottom: bool = False -> add a new line behind title
        """
        decorator = Text.setText(self, oneChar(decorator))
        spaces = (width-self._lenght)  
        if type(decorator)==type(Text('')):
            if align == 'center':
                if spaces%2==0:
                    title_text = f"{spaces//2*decorator.styled}"+f"{self.styled}"+f"{spaces//2*decorator.styled}"
                else:
                    title_text = f"{(spaces//2+1)*decorator.styled}"+f"{self.styled}"+f"{spaces//2*decorator.styled}"
            if align == 'right':
                title_text = f"{spaces*decorator.styled}"+f"{self.styled}"
            if align == 'left':
                title_text = f"{self.styled}"+f"{spaces*decorator.styled}"
        
        title_text = title_text + '\x1b[0m'
        if padding_up and padding_bottom:
            print(f"{self.bg_rgb}{width*' '}")
            print(f"{title_text}")
            print(f"{self.bg_rgb}{width*' '}", end='')
        elif padding_up:
            print(f"{self.bg_rgb}{width*' '}")
            print(f"{title_text}", end='')
        elif padding_bottom:
            print(f"{title_text}")
            print(f"{self.bg_rgb}{width*' '}", end='')
        else: print(title_text, end='')

        print('\x1b[0m', end='')

    def setTitle(self, title):
        """
        This method transform a string object to a Title object
        """
        if type(title) != type(Title('')):
            title = Title(str(title), fg=self.fg, bg=self.bg)
        if type(title) == type(Title('')) and title.bg != not_bg and title.fg != not_fg:
            title = Title(title.text, bg=title.bg, fg=title.fg, 
                            bold=title.bold, italic=title.italic, underline=title.underline, 
                            blink=title.blink, reverse=title.reverse, crossed=title.crossed)
        if type(title) == type(Title('')) and title.bg == not_bg and title.fg != not_fg:
            title = Title(title.text, bg=self.bg, fg=title.fg, 
                            bold=title.bold, italic=title.italic, underline=title.underline, 
                            blink=title.blink, reverse=title.reverse, crossed=title.crossed)
        if type(title) == type(Title('')) and title.fg == not_fg and title.bg != not_bg:
            title = Title(title.text, fg=self.fg, bg=title.bg,
                            bold=title.bold, italic=title.italic, underline=title.underline, 
                            blink=title.blink, reverse=title.reverse, crossed=title.crossed)
        if type(title) == type(Title('')) and title.fg == not_fg and title.bg == not_bg:
            title = Title(title.text, fg=self.fg, bg=self.bg,
                            bold=title.bold, italic=title.italic, underline=title.underline, 
                            blink=title.blink, reverse=title.reverse, crossed=title.crossed)
        
        return title
    
def oneChar(string):
    """
    This funtion is for validate just one length string in print titles and others
    """
    if type(string) == type(Text('')):
        if string.lenght > 1:
            string = Text(' ', string.fg, string.bg, string.bold,
                            string.italic, string.underline, string.blink,
                            string.reverse, string.crossed)
            return string
        else:
            return string
    if str(string):
        if len(string)>1:
            return ' '
        else:
            return string




from ..colors import setColor, html_rgb_bg, html_rgb_fg

not_fg = '\x1b[39m'
not_bg = '\x1b[49m'

class Text():
    """
    This class allow you to format text, some formats and effects are not available
    in some terminal emulators as Kitty, Alacritty, etc...
    A full list of terminals will be show on https://github.com/FreireAlexander/PyeMenu
    properties:
        text: str
    ***** Text Style *****
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        bold: bool 
        italic: bool 
        underline: bool 
        blink: bool 
        reverse: bool 
        crossed: bool 
        
        PyeMenu version 1.0.1
    """
    
    def __init__(self, text: str,
                fg: str = not_fg, bg: str = not_bg,
                bold: bool = False, italic: bool = False, 
                underline: bool = False, blink: bool = False, 
                reverse: bool = False, crossed: bool = False):
        self.text = text
        self._text = ' '+text+' '
        self.fg = fg
        self.fg_rgb = html_rgb_fg(fg)
        self.bg = bg
        self.bg_rgb = html_rgb_bg(bg)
        self.bold = bold
        self.italic = italic
        self.blink = blink
        self.underline = underline
        self.reverse = reverse
        self.crossed = crossed
        self.lenght = len(self.text)
        self.styled = Text.setStyle(self, self.text, self.fg, self.bg, 
                                    self.bold, self.italic, self.underline, 
                                    self.blink, self.reverse, self.crossed)

    def setStyle(self, chars, fg: str = not_fg, bg: str = not_bg,
                bold: bool = False, italic: bool = False, underline: bool = False,
                blink: bool = False, reverse: bool = False, crossed: bool = False):
        """
        This method set the Style with colors and effect to texts
        """
        result = chars
        if bold:
            result = f"\x1b[1m" + result
        if italic:
            result = f"\x1b[3m" + result
        if underline:
            result = f"\x1b[4m" + result
        if blink:
            result = f"\x1b[5m" + result
        if reverse:
            result = f"\x1b[7m" + result
        if crossed:
            result = f"\x1b[9m" + result
        if fg != not_fg and bg != not_bg:
            result = f"\x1b[38;{setColor(fg)}\x1b[48;{setColor(bg)}" + result
        if fg == not_fg and bg == not_bg:
            result = f"\x1b[0m" + result
        if fg == not_fg and bg != not_bg:
            result = f"\x1b[48;{setColor(bg)}" + result
        if fg != not_fg and bg == not_bg:
            result = f"\x1b[38;{setColor(fg)}" + result
    
        result += '\x1b[0m'
        return result
    
    def setText(self, cursor):
        """
        This method transform a string object to a Text object
        """
        if type(cursor) != type(Text('')):
            cursor = Text(str(cursor), fg=self.fg, bg=self.bg)
        if type(cursor) == type(Text('')) and cursor.fg != not_fg and cursor.bg != not_bg:
            cursor = Text(cursor.text, fg=cursor.fg, bg=cursor.bg, 
                            bold=cursor.bold, italic=cursor.italic, underline=cursor.underline, 
                            blink=cursor.blink, reverse=cursor.reverse, crossed=cursor.crossed)
        if type(cursor) == type(Text('')) and cursor.bg == not_bg and cursor.fg != not_fg:
            cursor = Text(cursor.text, bg=self.bg, fg=cursor.fg,
                            bold=cursor.bold, italic=cursor.italic, underline=cursor.underline, 
                            blink=cursor.blink, reverse=cursor.reverse, crossed=cursor.crossed)
        if type(cursor) == type(Text('')) and cursor.fg == not_fg and cursor.bg != not_bg:
            cursor = Text(cursor.text, fg=self.fg, bg=cursor.bg, 
                            bold=cursor.bold, italic=cursor.italic, underline=cursor.underline, 
                            blink=cursor.blink, reverse=cursor.reverse, crossed=cursor.crossed)
        if type(cursor) == type(Text('')) and cursor.fg == not_fg and cursor.bg == not_bg:
            cursor = Text(cursor.text, fg=self.fg, bg=self.bg,
                            bold=cursor.bold, italic=cursor.italic, underline=cursor.underline, 
                            blink=cursor.blink, reverse=cursor.reverse, crossed=cursor.crossed)
        
        return cursor
        


from .texts import Text

not_fg = '\x1b[39m'
not_bg = '\x1b[49m'

class Checkbox(Text):
    """
    Generate a Checkbox with its label, this class extend from Text()
    some formats and effects are not available
    in some terminal emulators as Kitty, Alacritty, etc...
    A full list of terminals will be show on https://github.com/FreireAlexander/PyeMenu
    properties:
        text: str
        box: str=' ' -> Indicate if the checkbox is selected or not by the default is an empty space      
    ***** Text Style *****
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        bold: bool 
        italic: bool 
        underline: bool 
        blink: bool 
        reverse: bool 
        crossed: bool
        
    PyeMenu version 1.0.1
    """
    def __init__(self, label: str, box: str=' ', 
                fg: str = not_fg, bg: str = not_bg,
                bold: bool = False, italic: bool = False, underline: bool = False,
                blink: bool = False, reverse: bool = False, crossed: bool = False):
        super().__init__(label, fg, bg,
                bold, italic, underline, blink, reverse, crossed)
        self._text = ' '+label+' '
        self._lenght = len(self._text)
        self.styled = Text.setStyle(self, self._text, self.fg, self.bg, bold, italic, underline, blink, reverse, crossed)
        self.box = box
        self.print = f"{self.bg_rgb}{self.fg_rgb} [{self.box}{self.bg_rgb}]"\
                    +f"{self.bg_rgb}{self.styled}{self.bg_rgb} "\
                    +"\x1b[0m"
    
    def onSelect(self):
        """
        This method mark or unmark the checkbox
        when it's marked the box parameter is equal to * 
        """
        if self.box == ' ':
            self.box = '*'
            self.print = f"{self.bg_rgb}{self.fg_rgb} [{self.box}{self.bg_rgb}]"\
                    +f"{self.bg_rgb}{self.styled}{self.bg_rgb} "\
                    +"\x1b[0m"
        else:
            self.box = ' '
            self.print = f"{self.bg_rgb}{self.fg_rgb} [{self.box}{self.bg_rgb}]"\
                    +f"{self.bg_rgb}{self.styled}{self.bg_rgb} "\
                    +"\x1b[0m"
    
    def clear(self):
        """
        this method unmark the checkbox 
        """
        self.box = ' '
        self.print = f"{self.bg_rgb}{self.fg_rgb} [{self.box}{self.bg_rgb}]"\
                +f"{self.bg_rgb}{self.styled}{self.bg_rgb} "\
                +"\x1b[0m"


"""
    A simple framework for develop simple and really easy Text User Interface, 
    You can Count with this elements:
    1. Components. These are used inside the widgets, these are:
        1. Text
        2. Title
        3. Checkbox
        4. Entry
        5. Button
    2. Widgets. These could be use to get information and data
        1. Menu -> For just one selection Menu
        2. Checkboxlist -> For a checkbox list with boolean choice
        3. Form -> A Simple widget to create Forms
    there are also a set of tools that can be use into your TUI as
        1. clear_screen() -> For cleaning the entire screen
        2. getKeyboard    -> For getting the input keyboard from user
        3. setCursor      -> For setting up the cursor position in a list
    Also, this package contain an special Class call Colors
        1. Colors -> A class with a lot of colors in hexadecimal format
    
    This package was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*

    PyeMenu version 1.0.0
"""
from .components import Text, Title, Entry, Button, Checkbox
from .widgets import Menu, Checkboxlist, Form
from .tools import clear_screen, getKeyboard
from .colors import Colors





"""
    This Sub Package contain all wdigets elements for the
    Text User Interface PyeMenu has, these elements are:
    1. Menu -> For just one selection Menu
    2. Checkboxlist -> For a checkbox list with boolean choice
    3. Form -> A Simple widget to create Forms

    This package was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
    
    PyeMenu version 1.0.1
"""
from .menus import Menu
from .checkboxlists import Checkboxlist
from .forms import Form



from setuptools import setup, find_packages
with open("README.md", "r", encoding= "utf-8") as fh:
  long_description = fh.read()

setup(
    name = 'pyemenu',         # How you named your package folder (MyLib)
    packages=find_packages(),
    version = '1.0.2',      # Start with a small number and increase it with every change you make
    author = 'Freire Alexander Palomino Palma',                   # Type in your name
    author_email = 'freirealexander0214@gmail.com',      # Type in your E-Mail
    license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description = 'Easy and Simple kit for develop Text User Interfaces',   # Give a short description about your library
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/FreireAlexander/PyeMenu',   # Provide either the link to your github or to your website
    download_url = 'https://github.com/FreireAlexander/PyeMenu/releases/tag/1.0.2',
    project_urls = {
        "Bug Tracker": "https://github.com/FreireAlexander/PyeMenu/issues",
    },
    #download_url = 'https://github.com/user/reponame/archive/v_01.tar.gz',    # I explain this later on
    keywords = ['MENU', 'CLI', 'command line', 'TUI', 'Text User Interface'],   # Keywords that define your package best
    install_requires=[            # I get to this in a second
            'readchar',
        ],
    classifiers=[
      'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
      'Intended Audience :: Developers',      # Define that your audience are developers
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',   # Again, pick a license
      'Programming Language :: Python :: 3.7',      #Specify which pyhton versions that you want to support
      'Programming Language :: Python :: 3.8',
      'Programming Language :: Python :: 3.9',
      'Programming Language :: Python :: 3.10',
      'Programming Language :: Python :: 3.11',
    ],
    python_requires = ">=3.7"
)


"""
    This Sub Package contain all components for being used insides widgets
    for develop a Text User Interface with PyeMenu, these components are:
        1. Text
        2. Title
        3. Checkbox
        4. Entry
        5. Button
    
    PyeMenu version 1.0.1
    This package was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
"""
from .texts import Text
from .titles import Title
from .checkboxs import Checkbox
from .entries import Entry
from .buttons import Button


from .texts import Text
from ..colors import Colors, html_rgb_bg

not_fg = '\x1b[39m'
not_bg = '\x1b[49m'

class Button(Text):
    """
    Generate a Button with its label, this class extend from Text()
    some formats and effects are not available
    in some terminal emulators as Kitty, Alacritty, etc...
    A full list of terminals will be show on https://github.com/FreireAlexander/PyeMenu
    properties:
        label: str         
    ***** Text Style *****
        fg: str -> A color in hexadecimal, much of colors could be found in Colors class 
        bg: str -> A color in hexadecimal, much of colors could be found in Colors class
        bold: bool 
        italic: bool 
        underline: bool 
        blink: bool 
        reverse: bool 
        crossed: bool
        
    PyeMenu version 1.0.1
    """
    def __init__(self, label: str,
                fg: str = Colors.DimGray, bg: str = Colors.BlackOlive,
                bold: bool = False, italic: bool = False, underline: bool = False,
                blink: bool = False, reverse: bool = False, crossed: bool = False):
        super().__init__(label, fg, bg,
                            bold, italic, underline, 
                            blink, reverse, crossed)
        self._text = '['+label+']'
        self._lenght = len(self._text)
        self.noFocus = Text.setStyle(self, self._text, self.fg, self.bg, bold, italic, underline, blink, reverse, crossed)
        self.print = f"{self.noFocus}"
        self.onClick = None

    def click(self):
        """
        This method activate the click mode for the Button
        """
        if self.onClick:
            self.onClick()
    